import 'dart:convert';
import 'dart:developer';
import 'package:flutter/material.dart';
import 'package:flutter_bloc/flutter_bloc.dart';
import 'package:hub/backend/schema/enums/enums.dart';
import 'package:hub/components/atomic_components/menu_button_item/menu_button_item_widget.dart';
import 'package:hub/components/atomic_components/menu_card_item/menu_card_item.dart';
import 'package:hub/components/molecular_components/menu_item/menu_item.dart';
import 'package:hub/flutter_flow/custom_functions.dart';
import 'package:hub/flutter_flow/flutter_flow_util.dart';
import 'package:hub/flutter_flow/nav/nav.dart';
import 'package:hub/shared/extensions/dialog_extensions.dart';
import 'package:hub/shared/helpers/license/license_helper.dart';

class MenuEvent {}

class MenuState {
  final List<MenuEntry?> menuEntries;
  final bool isGrid;

  MenuState({this.menuEntries = const [], this.isGrid = false});

  MenuState copyWith({List<MenuEntry?>? menuEntries, bool? isGrid}) {
    return MenuState(
      menuEntries: menuEntries ?? this.menuEntries,
      isGrid: isGrid ?? this.isGrid,
    );
  }
}

class MenuBloc extends Bloc<MenuEvent, MenuState> {
  final MenuView style;
  final MenuItem item;
  final bool expandable;
  final List<Module> menuOptions;

  MenuBloc({required this.style, required this.item, required this.expandable, required this.menuOptions}) : super(MenuState()) {
    on<MenuEvent>((event, emit) async {
      final entries = await generateMenuEntries();
      emit(state.copyWith(menuEntries: entries));
    });
  }
  Future<MenuEntry?> addMenuEntry(List<MenuEntry?> entries, IconData icon, String text, Function() action) async {
    entries.add(
      item == MenuItem.button
          ? MenuButtonWidget(icon: icon, action: action, title: text)
          : item == MenuItem.card
              ? MenuCardItem(icon: icon, action: action, title: text)
              : item == MenuItem.tile
                  ? MenuCardItem(icon: icon, action: action, title: text)
                  : null,
    );
    return null;
  }

  Future<List<MenuEntry?>> generateMenuEntries() async {
    List<MenuEntry?> entries = [];
    try {
      for (var opt in menuOptions) {
        String? licenseValue = await LicenseHelper().g(opt.value);
        if (licenseValue != null) {
          Map<String, String> licenseMap = await stringToMap(licenseValue);
          if (opt == Module.aboutProperty) log('AboutProperty: $licenseMap');
          log('Module: ${opt.value} - License: ${licenseMap['display']}');
          final String display = licenseMap['display'] ?? 'INVISIVEL';
          final String startDate = licenseMap['startDate'] ?? '';
          final String expirationDate = licenseMap['expirationDate'] ?? '';
          final bool isStarted = await processStartDate(startDate, opt);
          log('isStarted: $isStarted');
          final bool isExpired = await processExpirationDate(expirationDate, opt);
          log('isExpired: $isExpired');
          if (isStarted && !isExpired) {
            await processDisplay(display, opt, entries);
          }
        }
      }
    } catch (e) {
      log('Error generating menu entries: $e');
    }
    return entries;
  }

  Future<void> processDisplay(String display, Module opt, List<MenuEntry?> entries) async {
    try {
      switch (display) {
        case 'VISIVEL':
          await addMenuEntry(entries, opt.icon, opt.name, () async {
            await nav(opt.route);
          });
          break;
        case 'DESABILITADO':
          await addMenuEntry(entries, opt.icon, opt.name, () async {
            await DialogUnavailable.unavailableFeature(key.currentContext!);
          });
          break;
        case 'INVISIVEL':
        default:
          break;
      }
    } catch (e) {
      log('Error processing display for module ${opt.value}: $e');
    }
  }

  Future<bool> processStartDate(String startDate, Module opt) async {
    try {
      if (startDate.isEmpty) return true;
      final DateTime? start = DateTime.tryParse(startDate);
      if (start != null) {
        return DateTime.now().isAfter(start);
      }
    } catch (e) {
      log('Error processing start date for module ${opt.value}: $e');
    }
    return false;
  }

  Future<bool> processExpirationDate(String expirationDate, Module opt) async {
    try {
      if (expirationDate.isEmpty) return false;
      final DateTime? expiration = DateTime.tryParse(expirationDate);
      if (expiration != null) {
        return DateTime.now().isAfter(expiration);
      }
    } catch (e) {
      log('Error processing expiration date for module ${opt.value}: $e');
    }
    return false;
  }

  Future nav(String link) async {
    log('Opening: $link');
    key.currentContext!.push(link, extra: <String, dynamic>{
      kTransitionInfoKey: const TransitionInfo(
        hasTransition: false,
        transitionType: PageTransitionType.scale,
        alignment: Alignment.bottomCenter,
      ),
    });
  }
}
